@@ -104,6 +104,7 @@ class DocsService(gdata.service.GDataService):
     gdata.service.GDataService.__init__(
         self, email=email, password=password, service='writely', source=source,
         server=server, additional_headers=additional_headers, **kwargs)
+    self.ssl = True
 
   def _MakeKindCategory(self, label):
     if label is None:
@@ -172,6 +173,12 @@ class DocsService(gdata.service.GDataService):
     """
     server_response = self.request('GET', uri)
     response_body = server_response.read()
+    timeout = 5
+    while server_response.status == 302 and timeout > 0:
+      server_response = self.request('GET',
+                                     server_response.getheader('Location'))
+      response_body = server_response.read()
+      timeout -= 1
     if server_response.status != 200:
       raise gdata.service.RequestError, {'status': server_response.status,
                                          'reason': server_response.reason,
